@@ -5,16 +5,20 @@ use mio::{Poll, Event, Events, Token, Ready, PollOpt};
 use mio_extras::channel as mio_channel;
 use std::{collections::HashMap, time::Duration};
 use std::{
+  rc::Rc,
   sync::{Arc, RwLock},
 };
 
 use crate::{
-  dds::{message_receiver::MessageReceiver, reader::Reader, writer::Writer},
+  dds::{message_receiver::MessageReceiver}, 
+  dds::reader::{Reader, ReaderIngredients,}, 
+  dds::writer::{Writer, WriterIngredients,},
   network::util::get_local_multicast_locators,
   structure::builtin_endpoint::{BuiltinEndpointSet, },
   dds::qos::policy,
 };
 use crate::network::udp_listener::UDPListener;
+use crate::network::udp_sender::UDPSender;
 use crate::network::constant::*;
 use crate::structure::guid::{GuidPrefix, GUID, EntityId, TokenDecode};
 use crate::structure::entity::RTPSEntity;
@@ -27,7 +31,7 @@ use crate::{
   common::timed_event_handler::{TimedEventHandler},
   discovery::discovery_db::DiscoveryDB,
   structure::{dds_cache::DDSCache, topic_kind::TopicKind},
-  messages::submessages::submessages::AckNack,
+  messages::submessages::submessages::AckSubmessage,
 };
 
 use super::{
@@ -56,49 +60,50 @@ pub struct DPEventLoop {
   message_receiver: MessageReceiver, // This contains our Readers
 
   // Adding readers
-  add_reader_receiver: TokenReceiverPair<Reader>,
+  add_reader_receiver: TokenReceiverPair<ReaderIngredients>,
   remove_reader_receiver: TokenReceiverPair<GUID>,
   reader_timed_event_receiver: HashMap<Token, mio_channel::Receiver<TimerMessageType>>,
 
   // Writers
-  add_writer_receiver: TokenReceiverPair<Writer>,
+  add_writer_receiver: TokenReceiverPair<WriterIngredients>,
   remove_writer_receiver: TokenReceiverPair<GUID>,
   writer_timed_event_reciever: HashMap<Token, mio_channel::Receiver<TimerMessageType>>,
 
   stop_poll_receiver: mio_channel::Receiver<()>,
   // GuidPrefix sent in this channel needs to be RTPSMessage source_guid_prefix. Writer needs this to locate RTPSReaderProxy if negative acknack.
-  ack_nack_reciever: mio_channel::Receiver<(GuidPrefix, AckNack)>,
+  ack_nack_reciever: mio_channel::Receiver<(GuidPrefix, AckSubmessage)>,
 
   writers: HashMap<EntityId, Writer>,
+  udp_sender: Rc<UDPSender>,
 
   discovery_update_notification_receiver: mio_channel::Receiver<DiscoveryNotificationType>,
-
 }
 
 impl DPEventLoop {
   // This pub(crate) , because it should be constructed only by DomainParticipant.
+  #[allow(clippy::too_many_arguments)]
   pub(crate) fn new(
     domain_info: DomainInfo,
     udp_listeners: HashMap<Token, UDPListener>,
     ddscache: Arc<RwLock<DDSCache>>,
     discovery_db: Arc<RwLock<DiscoveryDB>>,
     participant_guid_prefix: GuidPrefix,
-    add_reader_receiver: TokenReceiverPair<Reader>,
+    add_reader_receiver: TokenReceiverPair<ReaderIngredients>,
     remove_reader_receiver: TokenReceiverPair<GUID>,
-    add_writer_receiver: TokenReceiverPair<Writer>,
+    add_writer_receiver: TokenReceiverPair<WriterIngredients>,
     remove_writer_receiver: TokenReceiverPair<GUID>,
     stop_poll_receiver: mio_channel::Receiver<()>,
     discovery_update_notification_receiver: mio_channel::Receiver<DiscoveryNotificationType>,
   ) -> DPEventLoop {
     let poll = Poll::new().expect("Unable to create new poll.");
     let (acknack_sender, acknack_reciever) =
-      mio_channel::sync_channel::<(GuidPrefix, AckNack)>(100);
+      mio_channel::sync_channel::<(GuidPrefix, AckSubmessage)>(100);
     let mut udp_listeners = udp_listeners;
     for (token, listener) in &mut udp_listeners {
       poll
         .register(
           listener.mio_socket(),
-          token.clone(),
+          *token,
           Ready::readable(),
           PollOpt::edge(),
         )
@@ -108,7 +113,7 @@ impl DPEventLoop {
     poll
       .register(
         &add_reader_receiver.receiver,
-        add_reader_receiver.token.clone(),
+        add_reader_receiver.token,
         Ready::readable(),
         PollOpt::edge(),
       )
@@ -117,7 +122,7 @@ impl DPEventLoop {
     poll
       .register(
         &remove_reader_receiver.receiver,
-        remove_reader_receiver.token.clone(),
+        remove_reader_receiver.token,
         Ready::readable(),
         PollOpt::edge(),
       )
@@ -167,6 +172,9 @@ impl DPEventLoop {
       )
       .expect("Failed to register reader update notification.");
 
+    // port number 0 menas OS chooses an available port number.
+    let udp_sender = UDPSender::new(0)
+      .expect("UDPSender construction fail"); // TODO
 
     DPEventLoop {
       domain_info,
@@ -174,6 +182,7 @@ impl DPEventLoop {
       ddscache,
       discovery_db,
       udp_listeners,
+      udp_sender: Rc::new(udp_sender),
       message_receiver: MessageReceiver::new(participant_guid_prefix, acknack_sender),
       add_reader_receiver,
       remove_reader_receiver,
@@ -309,9 +318,13 @@ impl DPEventLoop {
     match event.token() {
       ADD_READER_TOKEN => {
         trace!("add reader(s)");
-        while let Ok(mut new_reader) = self.add_reader_receiver.receiver.try_recv() {
+        while let Ok(new_reader_ing) = self.add_reader_receiver.receiver.try_recv() {
           let (timed_action_sender, timed_action_receiver) =
             mio_channel::sync_channel::<TimerMessageType>(10);
+
+          let mut new_reader = 
+            Reader::new(new_reader_ing, self.ddscache.clone(), self.udp_sender.clone());
+
           let time_handler: TimedEventHandler = TimedEventHandler::new(timed_action_sender.clone());
           new_reader.add_timed_event_handler(time_handler);
           // Timed action polling
@@ -367,13 +380,16 @@ impl DPEventLoop {
   fn handle_writer_action(&mut self, event: &Event) {
     match event.token() {
       ADD_WRITER_TOKEN => {
-        while let Ok(mut new_writer) = self.add_writer_receiver.receiver.try_recv() {
-          &self.poll.register(
+        while let Ok(new_writer_ingredients) = self.add_writer_receiver.receiver.try_recv() {
+          let mut new_writer = Writer::new(
+            new_writer_ingredients, self.ddscache.clone(), self.udp_sender.clone() );
+
+          self.poll.register(
             &new_writer.writer_command_receiver,
             new_writer.get_entity_token(),
             Ready::readable(),
             PollOpt::edge(),
-          );
+          ).expect("Writer command channel registration failed!!");
           let (timed_action_sender, timed_action_receiver) =
             mio_channel::sync_channel::<TimerMessageType>(10);
           let time_handler: TimedEventHandler = TimedEventHandler::new(timed_action_sender.clone());
@@ -442,18 +458,18 @@ impl DPEventLoop {
   }
 
   fn handle_writer_acknack_action(&mut self, _event: &Event) {
-    while let Ok((acknack_sender_prefix, acknack_message)) = self.ack_nack_reciever.try_recv() {
+    while let Ok((acknack_sender_prefix, acknack_submessage)) = self.ack_nack_reciever.try_recv() {
       let writer_guid = GUID::new_with_prefix_and_id(
         self.domain_info.domain_participant_guid.guidPrefix,
-        acknack_message.writer_id,
+        acknack_submessage.writer_id(),
       );
       if let Some(found_writer) = self.writers.get_mut(&writer_guid.entityId) {
         if found_writer.is_reliable() {
-          found_writer.handle_ack_nack(acknack_sender_prefix, acknack_message)
+          found_writer.handle_ack_nack(acknack_sender_prefix, acknack_submessage)
         }
       } else {
         warn!(
-          "Couldn't handle acknack! did not find local rtps writer with GUID: {:x?}",
+          "Couldn't handle acknack/nackfrag! Did not find local RTPS writer with GUID: {:x?}",
           writer_guid
         );
         continue;
@@ -462,11 +478,9 @@ impl DPEventLoop {
   }
 
   fn update_participant(&mut self, participant_guid_prefix: GuidPrefix ) {
-    debug!("update_participant - begin for {:?}", participant_guid_prefix);
-
-    if participant_guid_prefix == self.domain_info.domain_participant_guid.guidPrefix {
-      info!("Own participant update");
-    } 
+    info!("update_participant {:?} myself={}", 
+      participant_guid_prefix, 
+      participant_guid_prefix == self.domain_info.domain_participant_guid.guidPrefix);
 
 
     {
@@ -668,7 +682,6 @@ mod tests {
   use mio::{Ready, PollOpt};
   use crate::{
     dds::with_key::datareader::ReaderCommand,
-    structure::entity::RTPSEntity,
     dds::qos::QosPolicies,
   };
   use crate::structure::dds_cache::DDSCache;
@@ -677,7 +690,7 @@ mod tests {
   #[test]
   fn dpew_add_and_remove_readers() {
     // Adding readers
-    let (sender_add_reader, receiver_add) = mio_channel::channel::<Reader>();
+    let (sender_add_reader, receiver_add) = mio_channel::channel::<ReaderIngredients>();
     let (sender_remove_reader, receiver_remove) = mio_channel::channel::<GUID>();
 
     let (_add_writer_sender, add_writer_receiver) = mio_channel::channel();
@@ -697,34 +710,35 @@ mod tests {
       participant_id: 0,
     };
 
-    let dp_event_loop = DPEventLoop::new(
-      domain_info,
-      HashMap::new(),
-      ddshc,
-      discovery_db,
-      GuidPrefix::default(),
-      TokenReceiverPair {
-        token: ADD_READER_TOKEN,
-        receiver: receiver_add,
-      },
-      TokenReceiverPair {
-        token: REMOVE_READER_TOKEN,
-        receiver: receiver_remove,
-      },
-      TokenReceiverPair {
-        token: ADD_WRITER_TOKEN,
-        receiver: add_writer_receiver,
-      },
-      TokenReceiverPair {
-        token: REMOVE_WRITER_TOKEN,
-        receiver: remove_writer_receiver,
-      },
-      stop_poll_receiver,
-      discovery_update_notification_receiver,
-    );
-
     let (sender_stop, receiver_stop) = mio_channel::channel::<i32>();
-    dp_event_loop
+    
+    let child = thread::spawn(move || {
+      let dp_event_loop = DPEventLoop::new(
+        domain_info,
+        HashMap::new(),
+        ddshc,
+        discovery_db,
+        GuidPrefix::default(),
+        TokenReceiverPair {
+          token: ADD_READER_TOKEN,
+          receiver: receiver_add,
+        },
+        TokenReceiverPair {
+          token: REMOVE_READER_TOKEN,
+          receiver: receiver_remove,
+        },
+        TokenReceiverPair {
+          token: ADD_WRITER_TOKEN,
+          receiver: add_writer_receiver,
+        },
+        TokenReceiverPair {
+          token: REMOVE_WRITER_TOKEN,
+          receiver: remove_writer_receiver,
+        },
+        stop_poll_receiver,
+        discovery_update_notification_receiver,
+      );
+      dp_event_loop
       .poll
       .register(
         &receiver_stop,
@@ -733,8 +747,8 @@ mod tests {
         PollOpt::edge(),
       )
       .expect("Failed to register receivers.");
-
-    let child = thread::spawn(move || DPEventLoop::event_loop(dp_event_loop));
+      dp_event_loop.event_loop()
+    });
 
     let n = 3;
 
@@ -748,19 +762,23 @@ mod tests {
       let (_reader_commander, reader_command_receiver) =
         mio_extras::channel::sync_channel::<ReaderCommand>(100);
 
-      let new_reader = Reader::new(
-        new_guid,
-        send,
+      let new_reader_ing = ReaderIngredients {
+        guid: new_guid,
+        notification_sender: send,
         status_sender,
-        Arc::new(RwLock::new(DDSCache::new())),
-        "test".to_string(),
-        QosPolicies::qos_none(),
-        reader_command_receiver,
-      );
+        topic_name: "test".to_string(),
+        qos_policy: QosPolicies::qos_none(),
+        data_reader_command_receiver: reader_command_receiver,
+      };
 
-      reader_guids.push(new_reader.get_guid().clone());
-      info!("\nSent reader number {}: {:?}\n", i, &new_reader);
-      sender_add_reader.send(new_reader).unwrap();
+      // let new_reader = Reader::new(
+      //   new_reader_ing, 
+      //   Arc::new(RwLock::new(DDSCache::new())), 
+      //   Rc::new(UDPSender::new_with_random_port()), );
+
+      reader_guids.push(new_reader_ing.guid);
+      info!("\nSent reader number {}: {:?}\n", i, &new_reader_ing);
+      sender_add_reader.send(new_reader_ing).unwrap();
       std::thread::sleep(Duration::new(0, 100));
     }
 
