@@ -23,8 +23,8 @@ use crate::dds::{
   participant::*,
   topic::*,
   qos::*,
-  reader::Reader,
-  writer::Writer,
+  reader::ReaderIngredients,
+  writer::WriterIngredients,
   with_key::datawriter::DataWriter as WithKeyDataWriter,
   no_key::datawriter::DataWriter as NoKeyDataWriter,
   with_key::datareader::DataReader as WithKeyDataReader,
@@ -80,7 +80,7 @@ impl Publisher {
     discovery_db: Arc<RwLock<DiscoveryDB>>,
     qos: QosPolicies,
     default_dw_qos: QosPolicies,
-    add_writer_sender: mio_channel::SyncSender<Writer>,
+    add_writer_sender: mio_channel::SyncSender<WriterIngredients>,
     discovery_command: mio_channel::SyncSender<DiscoveryCommand>,
   ) -> Publisher {
     Publisher {
@@ -371,7 +371,7 @@ struct InnerPublisher {
   discovery_db: Arc<RwLock<DiscoveryDB>>,
   my_qos_policies: QosPolicies,
   default_datawriter_qos: QosPolicies, // used when creating a new DataWriter
-  add_writer_sender: mio_channel::SyncSender<Writer>,
+  add_writer_sender: mio_channel::SyncSender<WriterIngredients>,
   discovery_command: mio_channel::SyncSender<DiscoveryCommand>,
 }
 
@@ -382,7 +382,7 @@ impl InnerPublisher {
     discovery_db: Arc<RwLock<DiscoveryDB>>,
     qos: QosPolicies,
     default_dw_qos: QosPolicies,
-    add_writer_sender: mio_channel::SyncSender<Writer>,
+    add_writer_sender: mio_channel::SyncSender<WriterIngredients>,
     discovery_command: mio_channel::SyncSender<DiscoveryCommand>,
   ) -> InnerPublisher {
     InnerPublisher {
@@ -428,17 +428,17 @@ impl InnerPublisher {
               .or_else (|e| log_and_err_internal!("Where is my DomainParticipant? {}",e))?;
 
     let guid = GUID::new_with_prefix_and_id(dp.get_guid().guidPrefix, entity_id);
-    let new_writer = Writer::new(
-      guid.clone(),
-      hccc_download,
-      dp.get_dds_cache(),
-      topic.get_name().to_string(),
-      writer_qos,
-      status_sender,
-    );
+
+    let new_writer = WriterIngredients {
+        guid,
+        writer_command_receiver: hccc_download,
+        topic_name: topic.get_name(),
+        qos_policies: writer_qos,
+        status_sender,
+      };
 
     self.add_writer_sender.send(new_writer)
-      .or_else(|e| log_and_err_internal!("Adding new writer failed: {}",e))?;
+      .or_else(|e| log_and_err_internal!("Adding a new writer failed: {}",e))?;
 
     let data_writer = WithKeyDataWriter::<D, SA>::new(
           outer.clone(),
@@ -477,7 +477,7 @@ impl InnerPublisher {
     Ok(NoKeyDataWriter::<D, SA>::from_keyed(d))
   }
 
-  fn add_writer(&self, writer: Writer) -> Result<()> {
+  fn add_writer(&self, writer: WriterIngredients) -> Result<()> {
     match self.add_writer_sender.send(writer) {
       Ok(_) => Ok(()),
       _ => Err(Error::OutOfResources),
@@ -572,7 +572,7 @@ impl Subscriber {
     domain_participant: DomainParticipantWeak,
     discovery_db: Arc<RwLock<DiscoveryDB>>,
     qos: QosPolicies,
-    sender_add_reader: mio_channel::SyncSender<Reader>,
+    sender_add_reader: mio_channel::SyncSender<ReaderIngredients>,
     sender_remove_reader: mio_channel::SyncSender<GUID>,
     discovery_command: mio_channel::SyncSender<DiscoveryCommand>,
   ) -> Subscriber {
@@ -800,7 +800,7 @@ pub struct InnerSubscriber {
   domain_participant: DomainParticipantWeak,
   discovery_db: Arc<RwLock<DiscoveryDB>>,
   qos: QosPolicies,
-  sender_add_reader: mio_channel::SyncSender<Reader>,
+  sender_add_reader: mio_channel::SyncSender<ReaderIngredients>,
   sender_remove_reader: mio_channel::SyncSender<GUID>,
   discovery_command: mio_channel::SyncSender<DiscoveryCommand>,
 }
@@ -810,7 +810,7 @@ impl InnerSubscriber {
     domain_participant: DomainParticipantWeak,
     discovery_db: Arc<RwLock<DiscoveryDB>>,
     qos: QosPolicies,
-    sender_add_reader: mio_channel::SyncSender<Reader>,
+    sender_add_reader: mio_channel::SyncSender<ReaderIngredients>,
     sender_remove_reader: mio_channel::SyncSender<GUID>,
     discovery_command: mio_channel::SyncSender<DiscoveryCommand>,
   ) -> InnerSubscriber {
@@ -846,7 +846,7 @@ impl InnerSubscriber {
       mio_channel::sync_channel::<ReaderCommand>(4);
 
 
-    let qos = optional_qos.unwrap_or_else(|| topic.get_qos().clone());
+    let qos = optional_qos.unwrap_or_else(|| topic.get_qos());
 
     let entity_id = unwrap_or_random_EntityId(entity_id_opt, EntityKind::READER_WITH_KEY_USER_DEFINED);
 
@@ -861,15 +861,21 @@ impl InnerSubscriber {
 
     let reader_guid = GUID::new_with_prefix_and_id(dp.get_guid_prefix(), reader_id);
 
-    let new_reader = Reader::new(
-      reader_guid,
-      send,
+    let new_reader = ReaderIngredients {
+      guid: reader_guid,
+      notification_sender: send,
       status_sender,
-      dp.get_dds_cache(),
-      topic.get_name().to_string(),
-      qos.clone(),
-      reader_command_receiver,
-    );
+      topic_name: topic.get_name(),
+      qos_policy: qos.clone(),
+      data_reader_command_receiver: reader_command_receiver,
+    };
+
+    {
+      let mut db = self.discovery_db.write()
+                .or_else(|e| log_and_err_internal!("Cannot lock discovery_db. {}",e))?;
+      db.update_local_topic_reader(&dp, &topic, &new_reader);
+      db.update_topic_data_p(&topic);
+    }
 
     let datareader = WithKeyDataReader::<D, SA>::new(
       outer.clone(),
@@ -883,18 +889,11 @@ impl InnerSubscriber {
       reader_command_sender,
     )?;
 
-    {
-      let mut db = self.discovery_db.write()
-                .or_else(|e| log_and_err_internal!("Cannot lock discovery_db. {}",e))?;
-      db.update_local_topic_reader(&dp, &topic, &new_reader);
-      db.update_topic_data_p(&topic);
-    }
-
     // Create new topic to DDScache if one isn't present
     match dp.get_dds_cache().write() {
       Ok(mut dds_cache) => {
         dds_cache.add_new_topic(
-            &topic.get_name().to_string(),
+            &topic.get_name(),
             topic.kind(),
             topic.get_type(),
           );                   
