@@ -22,15 +22,6 @@
 #![allow(non_snake_case)]
 #![allow(non_upper_case_globals)]
 
-// extern crate bit_set;
-// extern crate bit_vec;
-extern crate bytes;
-extern crate num_derive;
-extern crate num_traits;
-extern crate speedy;
-extern crate tokio_util;
-extern crate uuid;
-
 #[macro_use]
 mod serialization_test;
 #[macro_use]
