@@ -9,14 +9,9 @@ use mio_extras::channel as mio_channel;
 
 use crate::structure::guid::{GUID,EntityKind};
 
-// Thease are the fixed poll tokens.
-// These are decoded using bitmaps, so the number of fixed tokens
-// is limited to 64 (0..63)
-// See GUID module for decoder details.
-
 const PTB : usize = EntityKind::POLL_TOKEN_BASE;
 
-pub const STOP_POLL_TOKEN: Token = Token(0 + PTB);
+pub const STOP_POLL_TOKEN: Token = Token(PTB);
 
 pub const DISCOVERY_SENDER_TOKEN: Token = Token(1 + PTB);
 pub const USER_TRAFFIC_SENDER_TOKEN: Token = Token(2 + PTB);
@@ -103,6 +98,7 @@ pub enum TimerMessageType {
 }
 
 #[derive(Debug)]
+#[allow(clippy::large_enum_variant)]
 pub(crate) enum DiscoveryNotificationType {
   ReaderUpdated { 
     rtps_reader_proxy: RtpsReaderProxy,
