@@ -4,6 +4,9 @@ use std::hash::Hash;
 //use crate::messages::fragment_number::FragmentNumber;
 use std::collections::BTreeSet;
 use std::cmp::{min,Ord,PartialOrd};
+use std::ops::Bound;
+use std::ops::RangeBounds;
+
 use num_derive::{FromPrimitive, NumOps, ToPrimitive};
 use speedy::{Context, Readable, Reader, Writable, Writer};
 use std::{convert::From};
@@ -93,6 +96,16 @@ impl Iterator for SequenceNumberRange {
   }
 }
 
+impl RangeBounds<SequenceNumber> for SequenceNumberRange
+{
+  fn start_bound(&self) -> Bound<&SequenceNumber> {
+    Bound::Included(&self.begin)
+  }
+  fn end_bound(&self) -> Bound<&SequenceNumber> {
+    Bound::Included(&self.end)
+  }
+}
+
 mod sequence_number_checked {
   use super::*;
   checked_impl!(CheckedAdd, checked_add, SequenceNumber);
